BODY {
  background: white;
  color: black;
  font-family: arial,sans-serif;
  margin: 0;
  padding: 1ex;
}

TABLE {
  border-collapse: collapse;
  border-spacing: 0;
  border-width: 0;
  color: inherit;
}

TD {
  margin: 0;
  padding: 0;
}

DIV {  border-width: 0; }
DT { margin-top: 1em; }

TH {
  background: #bbbbbb;
  color: inherit;
  padding: 0.4ex 1ex;
  text-align: left;
}

TH A:link, TH A:visited {
  background: transparent;
  color: black;
}

PRE     {
  background: #dddddd;
  border: 1px solid black;
  color: black;
  padding-top: 1em;
  padding-bottom: 1em;
  white-space: pre;
}

H1      {
  background: transparent;
  color: #006699;
  font-size: large;
}

H2      {
  background: transparent;
  color: #006699;
  font-size: medium;
}

LI {
  line-height: 1.2em;
  list-style-type: none;
}
 