<s snum=0001> 2 .  </s>
<s snum=0002> 2 .  </s>
<s snum=0003> oh , oh !  </s>
<s snum=0004> oh , oh !  </s>
<s snum=0005> oh , oh !  </s>
<s snum=0006> souvenons - nous , monsieur le Orateur , que ce sont ces secteurs de notre société qui servent de épine dorsale à notre économie .  </s>
<s snum=0007> bravo !  </s>
<s snum=0008> bravo !  </s>
<s snum=0009> monsieur le Orateur , ma question se adresse à le ministre chargé de les transports .  </s>
<s snum=0010> tous deux possèdent de nombreuses années de expérience dans la fabrication et la distribution de les produits forestiers .  </s>
<s snum=0011> 1 .  </s>
<s snum=0012> je voudrais aussi parler de le droit de le fournisseur de choisir ses clients à son gré .  </s>
<s snum=0013> je en ai assez de voir que les ministres semblent éprouver certains complexes à le égard de la négociation collective .  </s>
<s snum=0014> je voudrais commenter un autre point particulier , monsieur le Orateur , avant de me taire .  </s>
<s snum=0015> a le ordre .  </s>
<s snum=0016> 3 .  </s>
<s snum=0017> il ne y a pas de meilleure autorité en la matière que le député de Don Valley , non pas moi exclusivement .  </s>
<s snum=0018> ce est tout à fait compréhensible .  </s>
<s snum=0019> en outre , elle pourrait constituer une sérieuse menace pour la Confédération et le unité nationale .  </s>
<s snum=0020> pour cette raison , nous ne avons pas cru bon de assister à la réunion et en avons informé le COJO en conséquence .  </s>
<s snum=0021> monsieur le Orateur , le ministre a laissé entendre que un programme de construction plus considérable peut - être une poussée inflationniste .  </s>
<s snum=0022> je ne partage pas le avis de le ministre .  </s>
<s snum=0023> il y a beaucoup à dire sur les deux aspects de cette question .  </s>
<s snum=0024> a le ordre .  </s>
<s snum=0025> monsieur le Orateur , comme je le ai signalé à le chef de le opposition , je espère faire une déclaration à ce sujet le1er novembre .  </s>
<s snum=0026> ce est bien cela .  </s>
<s snum=0027> ce ne est pas en isolant quelque un que on le aide à se réadapter .  </s>
<s snum=0028> il laissait une femme et une famille .  </s>
<s snum=0029> monsieur le président je invoque le Règlement ; je suis persuadé que le député de Verdun ne aurait pas déprécié ma position .  </s>
<s snum=0030> les compagnies de assurance peuvent - elles à leur tour , agir comme compagnies aériennes ?  </s>
<s snum=0031> je vous le déclare , monsieur le Orateur , la population de Toronto sait que le gouvernement est incapable de fournir une réponse .  </s>
<s snum=0032> on a tout à fait tenu compte de leur avis sur le état de la réserve de poisson .  </s>
<s snum=0033> je vous remercie , monsieur le Orateur , et je remercie les députés de me permettre de poursuivre .  </s>
<s snum=0034> je réclame instamment cette étude approfondie , monsieur le Orateur , car nous ne savons presque rien de les raisons qui transforment les gens en criminels .  </s>
<s snum=0035> le gouvernement doit aider ses propres employés qui peuvent avoir besoin de une autre langue pour leur avancement .  </s>
<s snum=0036> je ne ai pas de scrupule quant à le genre de langage que je tiens .  </s>
<s snum=0037> ( la motion est adoptée . )  </s>
<s snum=0038> monsieur le président , la décision ne est pas prise à ce moment - ci , mais je imagine que ce est oui .  </s>
<s snum=0039> la question de les relations commerciales avec nos homologues européens devrait être envisagée très sérieusement .  </s>
<s snum=0040> ce dont nous , les Canadiens , devons nous soucier , ce est les cas où on exporte ces animaux dans ce but - là .  </s>
<s snum=0041> ils ont déjà indiqué les embranchements que ils songeaient à fermer .  </s>
<s snum=0042> ce ministre assurait la garde tandis que la moitié de les recettes et de les bénéfices de le industrie agricole canadienne étaient balayés par le exposé budgétaire .  </s>
<s snum=0043> en aucune façon , monsieur le président .  </s>
<s snum=0044> nous en faisons actuellement le analyse et je espère que bientôt la Chambre pourra être saisie de les renseignements pertinents .  </s>
<s snum=0045> bravo !  </s>
<s snum=0046> bravo !  </s>
<s snum=0047> je proposerais donc , avec le appui de le député de Halifax - East ( M. Forrestall ) :  </s>
<s snum=0048> quand on a saisi le énormité de le ravage exercé par le crime organisé dans notre pays , on doit envisager de les solutions en profondeur .  </s>
<s snum=0049> leur popularité a monté en flèche et ils ont traversé la crise avec facilité grâce à cela .  </s>
<s snum=0050> ( 2 ) Un comité doit se en tenir à le ordre de renvoi et ne saurait y déroger .  </s>
<s snum=0051> je voudrais préciser un autre point , et je le fais parce que je estime que il se agit de un débat général .  </s>
<s snum=0052> en décembre , la Colombie - Britannique a fait savoir que elle allait se retirer de le Office canadien de commercialisation de les oeufs .  </s>
<s snum=0053> bien de les députés de tous les partis de ce côté - ci de la Chambre se y sont également fortement opposés .  </s>
<s snum=0054> ce ne est pas de ces lignes que il faudrait se inquiéter pour le instant .  </s>
<s snum=0055> le député a pu poser deux questions supplémentaires et , en toute justice , je pense que nous devrions laisser la parole à de autres députés .  </s>
<s snum=0056> je crois que ce est une bonne chose .  </s>
<s snum=0057> je crois que il a pris trop de temps à étudier de les demandes courantes de changements de installations le long de un pipe - line , par exemple .  </s>
<s snum=0058> le ministre de État chargé de les Sciences et de la Technologie devrait connaître son rayon .  </s>
<s snum=0059> les menuisiers de le gouvernement fédéral touchent $ 6.42 à Toronto contre $ 5.23 à Halifax et Moncton .  </s>
<s snum=0060> je aimerais mentionner à ce sujet un communiqué de presse qui a suivi la conférence fédérale - provinciale de les procureurs généraux de octobre 1975 .  </s>
<s snum=0061> monsieur le Orateur , je invoque le Règlement .  </s>
<s snum=0062> je pense que le terme ferroviaire est « chargement sur demande » .  </s>
<s snum=0063> je reviendrai plus tard à ce que coûte un bâtiment de pêche de nos jours .  </s>
<s snum=0064> dans la plupart de les provinces , de les règlements généraux sur le bruit ont été mis en application , et dont le principe général se énonce ainsi :  </s>
<s snum=0065> de accord .  </s>
<s snum=0066> après tout , le énergie est un bien collectif et le gouvernement pourrait fort bien adopter une politique de encouragement comme celle dont je ai parlé .  </s>
<s snum=0067> 6 .  </s>
<s snum=0068> je serai heureux , quand nous en aurons le temps , de me asseoir avec les député et de explorer la question de façon productive .  </s>
<s snum=0069> le ministre prendra il de les mesures pour résoudre les difficultés , car il est absolument nécessaire de passer les commandes maintenant et non dans quelques mois ?  </s>
<s snum=0070> cela encouragera la construction de nouvelles embarcations de pêche et la création de nouvelles conserveries .  </s>
<s snum=0071> il y a bien de les choses que je pourrais dire à propos de le bill C - 19 .  </s>
<s snum=0072> je reviendrai sur cet aspect un peu plus tard , car je pense que ce est un élément que il est essentiel de souligner cet après - midi .  </s>
<s snum=0073> les sociétés trouvent que les règlements sont compliqués , lourds et que on les modifie sans cesse .  </s>
<s snum=0074> monsieur le Orateur , ma question se adresse à le ministre chargé de la Commission canadienne de le blé .  </s>
<s snum=0075> si je comprends bien , on prévoit un taux de intérêts annuel de 70 % .  </s>
<s snum=0076> nous souhaitons parvenir à une décision cette semaine .  </s>
<s snum=0077> nous acceptons votre opinion .  </s>
<s snum=0078> il fit campagne là - dessus pendant les élections .  </s>
<s snum=0079> 12 .  </s>
<s snum=0080> monsieur le Orateur , voici ce que je ai à en dire : quelle cruelle comédie , car ce est jouer la comédie .  </s>
<s snum=0081> monsieur le Orateur , nous essaierons de nous en tenir à cette suggestion .  </s>
<s snum=0082> 1 . a ) $ 98,355 . b ) Ils sont disponibles dans tous les grands bureaux de poste de le pays .  </s>
<s snum=0083> en vertu de la constitution , le activité criminelle est de compétence fédérale , tandis que le application de la loi relève de les provinces et de les municipalités .  </s>
<s snum=0084> je prétends que mes remarques concernant le bill , le argent dépensé pour les soins médicaux et la façon dont on le dépense .  </s>
<s snum=0085> de abord quelques mots à le sujet de cette maladie pour vous situer .  </s>
<s snum=0086> heureusement , il y en a moins cette année , elles sont plus raisonnables , plus sérieuses et plus conciliantes .  </s>
<s snum=0087> ce projet de loi est à le étude à la Chambre depuis plus de un an .  </s>
<s snum=0088> cette personne a pris sa retraite le 30 décembre 1975 après avoir complété 34 années à le service de le public .  </s>
<s snum=0089> le adoption de la motion de le député signifierait que la loi préciserait en détail les exigences de contenu et que nous pourrions les discuter .  </s>
<s snum=0090> cela ne veut pas dire nécessairement que il y a défaut de se y conformer de la part de les parties .  </s>
<s snum=0091> je ne lui ai pas demandé de me fournir de telles explications sur ces activités .  </s>
<s snum=0092> comment peut - on tant demander de une part et être assuré de si peu de le autre ?  </s>
<s snum=0093> peut - être proposera il ensuite une mesure nouvelle et différente qui ne le exposera pas à les accusations de censure .  </s>
<s snum=0094> ou ils font leur travail convenablement , ou ce est à vous de obtenir le pouvoir nécessaire pour les amener à le faire .  </s>
<s snum=0095> on y retrouve en effet toutes les tensions et toutes les divergences de notre pays .  </s>
<s snum=0096> le hon. John N . Turner ( ministre de les Finances ) propose :  </s>
<s snum=0097> bien que il me coûte de le admettre , les gouvernements libéraux à le Canada ont présenté de bonnes mesures législatives ...  </s>
<s snum=0098> en tant que député de une circonscription voisine , je connais fort bien sa persévérance et son acharnement à le travail .  </s>
<s snum=0099> où est la CAI ?  </s>
<s snum=0100> malheureusement , à le heure actuelle , cette réalité est perçue par les deux tiers de la population non pas comme un avantage , mais comme un obstacle .  </s>
<s snum=0101> ne nous forcez pas à accepter un moratoire .  </s>
<s snum=0102> ce ne est pas la première fois que les députés en entendront parler .  </s>
<s snum=0103> on trouvait la même recommandation dans le rapport de le 29 avril de le an dernier de le comité permanent de les privilèges et élections .  </s>
<s snum=0104> je crois , monsieur le président , que le ministre est mal informé .  </s>
<s snum=0105> dans le rapport de le ministère de les Transports sur le accident , il est dit :  </s>
<s snum=0106> monsieur le Orateur , le gouvernement fédéral a offert un programme de 100 millions de dollars .  </s>
<s snum=0107> dans la plupart de les échantillons de cette importance , ce est - à - dire un groupe de plus de 500 personnes , cela serait considéré comme négligeable .  </s>
<s snum=0108> de ordinaire pour demander de nouveaux pouvoirs de emprunt ou annexer un article à les bills de finances provisoires présentés à la Chambre .  </s>
<s snum=0109> cela touche à le coeur même de notre démocratie puisque il se agit de le droit de le peuple à le information .  </s>
<s snum=0110> cela réduirait la demande de pétrole .  </s>
<s snum=0111> la Division de les enquêtes spéciales a été abolie à le début de 1975 .  </s>
<s snum=0112> de plus , il ne se agit pas de faire savoir si le on est pour ou contre une forme de aide à les parents adoptifs .  </s>
<s snum=0113> montrez - nous les documents .  </s>
<s snum=0114> le Canada continue à accorder son soutien à le alliance de le OTAN comme force de dissuasion et comme assurance absolument indispensables à notre sécurité .  </s>
<s snum=0115> pouvez - vous imaginer une telle mesure en 1978 ?  </s>
<s snum=0116> non , monsieur le président .  </s>
<s snum=0117> bien entendu , le gouvernement tente de conclure le meilleur marché possible pour les Canadiens en faisant une série de négociations .  </s>
<s snum=0118> je ajouterais , monsieur le Orateur , que il faut la maintenir à ce niveau .  </s>
<s snum=0119> si je veux obtenir de les chiffres sur la situation de le Canada , je ne irai certainement pas consulter le discours prononcé par le premier ministre .  </s>
<s snum=0120> nous avons étudié tous les aspects de la question .  </s>
<s snum=0121> mieux vaut sans doute que je cite un journaliste de Timmins qui songeait vraisemblablement à le député .  </s>
<s snum=0122> monsieur le Orateur , il est vrai que je ai reçu le rapport de le comité de action de les chemins de fer de les Prairies .  </s>
<s snum=0123> ( ii ) a ) 3 véhicules seront utilisés par 6 spécialistes canadiens dans le cadre de la prestation de aide technique .  </s>
<s snum=0124> 1979 .  </s>
<s snum=0125> et nous allons appliquer chacune de les recommandations contenues dans le rapport de le Vérificateur général , chose que nous réclamons depuis dix ans .  </s>
<s snum=0126> je voudrais commencer , monsieur le Orateur , en faisant quelques observations générales .  </s>
<s snum=0127> monsieur le Orateur , je vais voir si ces chiffres sont disponibles , mais je déplore les expressions que le député a utilisées .  </s>
<s snum=0128> je sais que il ne a pas le intention de me empêcher de répondre à la question fort sérieuse que me a posée son collègue .  </s>
<s snum=0129> le opposition nous chante toujours la même rengaine : ce programme coûtera de le argent à le gouvernement .  </s>
<s snum=0130> notre taxe de 18 cents ne leur plaisait pas .  </s>
<s snum=0131> le problème est de savoir si la population canadienne a le droit de être informée sur sa constitution .  </s>
<s snum=0132> ils veulent choisir eux - mêmes qui ils vont défendre .  </s>
<s snum=0133> est - ce que cela a été mis par écrit ?  </s>
<s snum=0134> je suis conscient , monsieur le président , que il ne existe pas de solution miracle .  </s>
<s snum=0135> en 1978 , on a enregistré 1,122,000 divorces sur le continent .  </s>
<s snum=0136> le chômage brise les familles et porte à le alcool et à la drogue .  </s>
<s snum=0137> le opposition officielle a essayé de détruire Petro - Canada pendant son bref règne de six mois .  </s>
<s snum=0138> et que à le lieu de provoquer la rupture ,  </s>
<s snum=0139> ils se sont prononcés contre une mesure qui aurait garanti à les Canadiens de les approvisionnements sûrs .  </s>
<s snum=0140> a propos de contribution , cela me rappelle une histoire .  </s>
<s snum=0141> madame le Président , je aimerais entendre ce que va dire le député avant de proposer ma motion .  </s>
<s snum=0142> le orateur qui me a précédé a dit également que il aimerait voir les provinces participer à les préparatifs de les réunions .  </s>
<s snum=0143> mon collègue de Toronto dit encore 53 ans .  </s>
<s snum=0144> de après toutes ces estimations , le Canada sera en bien plus mauvaise posture par suite de le programme que il ne le était auparavant .  </s>
<s snum=0145> monsieur le premier ministre ,  </s>
<s snum=0146> en fait , elle aura de les effets plus destructifs que positifs .  </s>
<s snum=0147> le inflation mondiale crée de graves tensions économiques .  </s>
<s snum=0148> le PT7 représente le début de une nouvelle famille de moteurs dans un niveau de puissance au - dessus de le moteur PT6 qui est hautement couronné de succès .  </s>
<s snum=0149> monsieur le Orateur , je suis heureux de avoir le occasion de participer à ce débat de ajournement ...  </s>
<s snum=0150> cette politique joue un rôle aussi important dans une stratégie industrielle que le plan de ensemble ou que les plans détaillés que il faut mettre à le point .  </s>
<s snum=0151> on dit maintenant que la construction de le gazoduc va coûter environ 45 milliards de dollars .  </s>
<s snum=0152> signalons par ailleurs que on trouve toutes ces lacunes dans le bill .  </s>
<s snum=0153> a ce stade de le débat sur la résolution , mon parti propose trois amendements précis .  </s>
<s snum=0154> il ne peut donc pas prétendre avoir apporté le moindre changement à le niveau de le ensemble .  </s>
<s snum=0155> ce est dans le cadre de ce programme et de ententes auxiliaires que le gouvernement accorde la majeure partie de son aide à le expansion régionale .  </s>
<s snum=0156> que a fait le gouvernement à ce sujet ?  </s>
<s snum=0157> si nous utilisons le chômage pour combattre le inflation , nous pourrons relancer le économie , voilà la position de le gouvernement .  </s>
<s snum=0158> la conservation commence à produire de le effet .  </s>
<s snum=0159> on pourrait implanter une aciérie en Colombie - Britannique , par exemple .  </s>
<s snum=0160> a le ordre !  </s>
<s snum=0161> voilà pourquoi les professions libérales ne devaient pas être imposées pour les travaux en cours .  </s>
<s snum=0162> monsieur le Président , je crois que ce qui se passe aujourd' hui est intéressant , mais ce ne est que un début .  </s>
<s snum=0163> cependant , ce sont les contribuables qui sont obligés de payer .  </s>
<s snum=0164> le projet de loi a pour but de revenir sur de les programmes qui ont été mis en place à une époque où nous pouvions nous le permettre .  </s>
<s snum=0165> ce est pourquoi nous avons dû importer 25,000 travailleurs spécialisés à un moment où le chômage augmentait .  </s>
<s snum=0166> comme Carter le a si bien dit dans les années 60 , une piastre est toujours une piastre , et il faut le imposer comme tel .  </s>
<s snum=0167> a le ordre , je vous prie .  </s>
<s snum=0168> cela donne en tout une réduction de $ 2,958 à le cours de les deux prochaines années , soit 15 % de le salaire initial de $ 20,000 .  </s>
<s snum=0169> de après les nouvelles de ce matin , la mine de minerai de fer à Heath , dans le nord de le Nouveau - Brunswick va fermer .  </s>
<s snum=0170> ils sont prêts à suivre cet exemple .  </s>
<s snum=0171> a le heure actuelle , un certain nombre de programmes de ce genre fonctionnent déjà et nous prévoyons en lancer de autres .  </s>
<s snum=0172> nous devons nous demander comment il se fait que cette abondance de idées et de activité ne ait pas résolu le problème de le développement .  </s>
<s snum=0173> je voudrais terminer sur une note positive , monsieur le Orateur .  </s>
<s snum=0174> je voudrais que il soit bien clair , monsieur le Orateur , que le gouvernement fédéral ne fixe pas les prix de détail .  </s>
<s snum=0175> la séance est ouverte à 2 heures .  </s>
<s snum=0176> je rappelle , monsieur le président , que le ministre parlait , pour les lignes de transmission de électricité , de les mêmes pouvoirs qui existent déjà .  </s>
<s snum=0177> ce est vraiment un objectif souhaitable que nous visons tous .  </s>
<s snum=0178> madame le Président , je voudrais poser une question supplémentaire à le ministre de les Transports .  </s>
<s snum=0179> prenons maintenant Petro - Canada .  </s>
<s snum=0180> cela va apporter de les économies énormes .  </s>
<s snum=0181> malheureusement , personne ne semble savoir quels seront à le juste ces effets .  </s>
<s snum=0182> nous avons permis à un sous - gouvernement de arracher de les pouvoirs à la Chambre de les communes .  </s>
<s snum=0183> ce est une somme considérable .  </s>
<s snum=0184> que se passe il exactement ?  </s>
<s snum=0185> madame le Président , je serai très bref .  </s>
<s snum=0186> le gouvernement dit tout simplement à les gens ce qui est bon pour eux .  </s>
<s snum=0187> ce sont les données désaisonnalisés , celles qui montrent une baisse de 5 % , qui comptent .  </s>
<s snum=0188> les discussions qui se déroulent en ce moment mettent la présidence dans une situation fort embarrassante .  </s>
<s snum=0189> monsieur le Président , je regrette , mais je ne ai pas vu ce communiqué de la société britannique .  </s>
<s snum=0190> il y a moins de bénévoles , car beaucoup de femmes sont retournées sur le marché de le travail .  </s>
<s snum=0191> elles travaillent avec eux .  </s>
<s snum=0192> leur succès dépend de la mesure dans laquelle ils se appliquent .  </s>
<s snum=0193> je ne sais pas se il existe un seul député à la Chambre qui se opposerait à ce que on donne davantage de argent à les personnes âgées .  </s>
<s snum=0194> eux aussi ont un rôle à jouer dans la chaîne alimentaire .  </s>
<s snum=0195> un aspect plus démocratique a été ajouté à les situations qui se présentaient dans nos circonscriptions .  </s>
<s snum=0196> il faudrait plutôt se efforcer de faire oublier les propos stupides que a tenus le ministre la semaine dernière .  </s>
<s snum=0197> le ministère remplira il le ravin qui se trouve à le extrémité de la piste 24L à le aéroport international Pearson de Toronto ?  </s>
<s snum=0198> si nous décidons de présenter un tel projet de loi , je lui rappellerai son offre .  </s>
<s snum=0199> a cela , je réponds :  </s>
<s snum=0200> ce est ce que je recommande à la Chambre de faire , monsieur .  </s>
<s snum=0201> je ignore si cela servira à quelque chose .  </s>
<s snum=0202> voilà le principe dont le Alliance doit se inspirer en cherchant à conclure de les accords justes et vérifiables avec le Union soviétique .  </s>
<s snum=0203> la même réponse se applique à les automobiles .  </s>
<s snum=0204> il a saisi la Chambre de un projet de loi qui aura pour effet de les restreindre .  </s>
<s snum=0205> le député de Bow River ( M. Taylor ) intervient dans le débat .  </s>
<s snum=0206> monsieur le Président , je invoque le Règlement .  </s>
<s snum=0207> le gouvernement ne veut pas le croire .  </s>
<s snum=0208> ce étaient là de les engagements extrêmement intéressants et assez révélateurs sur ce que pourrait devenir ce secteur fondamentalement important , le agriculture .  </s>
<s snum=0209> il se agit de le syndrome de la porte tambour et la réarrestation immédiate .  </s>
<s snum=0210> je espère que ensuite il portera le flambeau .  </s>
<s snum=0211> le argument de le gouvernement est erroné , à mon avis .  </s>
<s snum=0212> toutefois , nous avons suffisamment de argent pour renflouer deux banques .  </s>
<s snum=0213> le Règlement !  </s>
<s snum=0214> ces créations peuvent être volées comme ne importe quel autre type de propriété .  </s>
<s snum=0215> deuxièmement , monsieur le Président , ce que nous avons fait depuis novembre dernier , ce est une amélioration de le climat de confiance .  </s>
<s snum=0216> ce sont de les établissements financiers très importants dans le ouest de le Canada .  </s>
<s snum=0217> le député de Cochrane - Supérieur ( M. Penner ) a dit tout à le heure que ce était à le gouvernement de libérer ses députés .  </s>
<s snum=0218> ce est vrai .  </s>
<s snum=0219> que fait - on de les femmes qui vivent loin de les centres où ces services sont offerts ?  </s>
<s snum=0220> ils demandent à les députés de le Parlement de mettre fin à toute réduction de personnel dans ces ateliers .  </s>
<s snum=0221> ce ne est pas une question de me joindre à le député ; nous avons de le avance sur lui .  </s>
<s snum=0222> le Dr Kenneth Hare , considéré chez nous comme le un de nos spécialistes les plus éminents de les questions atmosphériques , a écrit un article où il précise :  </s>
<s snum=0223> cela aura de les répercussions un peu partout .  </s>
<s snum=0224> en de autres mots , les céréaliers américains toucheront une subvention de environ $ 2 le boisseau que leur versera le gouvernement de M. Reagan .  </s>
<s snum=0225> le rapport complet sera déposé de ici le automne prochain .  </s>
<s snum=0226> si ces règles étaient appliquées , est - ce que nous traiterions effectivement tout le monde sur le même pied ?  </s>
<s snum=0227> cette éminente autorité à la Chambre de les communes a parlé de harcèlements sexuels , de certaines femmes qui avaient été obligées de se dévêtir pour obtenir un emploi .  </s>
<s snum=0228> je ne fais que avertir les députés de le existence de un problème à cause de ce qui se est passé .  </s>
<s snum=0229> Évidemment , quand les conservateurs étaient dans le opposition , ils tenaient de les discours qui étaient à le opposé de leurs actions de aujourd' hui .  </s>
<s snum=0230> en réalité , le ministre de la Justice et le solliciteur général ont répondu avec tous les détails que ils pouvaient humainement donner .  </s>
<s snum=0231> le vice - premier ministre croit - il que ce est ainsi que un gouvernement conscient de ses responsabilités , ou un pays indépendant doit conclure un traité international ?  </s>
<s snum=0232> le sous - comité parlementaire est en train de examiner toute la question de les droits à le égalité .  </s>
<s snum=0233> en réalité , pour les demandes concernant moins de quatre étudiants , on ne essaie pas de savoir si les emplois sont axés sur la carrière ou non .  </s>
<s snum=0234> de plus , nous voulons une série de critères objectifs fondés sur les éléments que je mentionne ici , par exemple :  </s>
<s snum=0235> si nous adoptons pareille attitude pour cette question ou de autres de le domaine public , nous trouverons toujours de les excuses pour éviter de passer à le action .  </s>
<s snum=0236> vous avez alors instauré votre programme , le hôtel a été abandonné et Fort St. John est devenu une ville fantôme .  </s>
<s snum=0237> nous , en six mois , on a consulté , on se est assis avec le ministre et on est arrivé à une entente .  </s>
<s snum=0238> le très honorable député ne est peut - être pas non plus de accord avec le point de vue de Mme Bégin .  </s>
<s snum=0239> " essayons - le , disent - ils , et voyons ses effets .  </s>
<s snum=0240> le MEER fournit environ 3 millions de ce montant et le ministère de les Affaires indiennes et de le Nord le reste .  </s>
<s snum=0241> on ne répétera jamais trop que les Canadiens se inquiètent de la montée de la grande criminalité .  </s>
<s snum=0242> les exportations de grains doivent fléchir de 25 % pendant la campagne 1984 - 1985 .  </s>
<s snum=0243> notre retard dans ce domaine est si important que tous les Canadiens en sont gênés .  </s>
<s snum=0244> on ne modifie pas vraiment la situation .  </s>
<s snum=0245> il est désormais impossible de établir une distinction valable quelconque entre les missiles nucléaires de portée intermédiaire et de portée intercontinentale .  </s>
<s snum=0246> puis - je demander à le député de Western Arctic de être bref , si possible .  </s>
<s snum=0247> le Grand Victoria est considéré comme la Mecque de le tourisme à le Canada .  </s>
<s snum=0248> la taxe ne serait pas prélevée pour être remboursée par la suite .  </s>
<s snum=0249> on se est déjà engagé à faire en sorte que leur taux de participation se accroisse de manière à atteindre à le moins les niveaux historiques sinon traditionnels .  </s>
<s snum=0250> ce est - à - dire à la demande de la personne qui va peut - être prétendre à le statut de réfugié .  </s>
<s snum=0251> voilà où nous en sommes actuellement .  </s>
<s snum=0252> les paiements de péréquation ont acquis à le cours de les ans une ampleur considérable .  </s>
<s snum=0253> les représentants de tous les partis se inquiètent de cette question .  </s>
<s snum=0254> je tiens également à signaler que nous avons consulté le caucus progressiste conservateur de le Manitoba et un certain nombre de personnes de cette province .  </s>
<s snum=0255> la localité gardera un bon souvenir de le personnel militaire et de leur famille qui ont participé à la vie de cette localité .  </s>
<s snum=0256> en toute déférence , je dois interrompre le très honorable premier ministre .  </s>
<s snum=0257> alors , je le ai expliqué .  </s>
<s snum=0258> si le premier ministre a déclaré que il est prêt à négocier encore avec le Québec , pourquoi adopter en vapeur ce projet de loi ?  </s>
<s snum=0259> ce est la raison de ailleurs pour laquelle certains de mes collègues ont fait de les interventions relativement rigoureuses à ce sujet .  </s>
<s snum=0260> je tiens à exposer la teneur précise de les notes qui figurent dans le état financier de le 31 mars 1984 .  </s>
<s snum=0261> cette décision est alors envoyée à le particulier en question .  </s>
<s snum=0262> tout cela est insensé .  </s>
<s snum=0263> dans ce dossier , monsieur le Président , de nombreuses autorités se sont donné la main pour réagir immédiatement à la situation de urgence .  </s>
<s snum=0264> que est - il donc advenu de les études qui ont été effectuées après cet incident ?  </s>
<s snum=0265> le ex - ministre de les Transports avait clairement établi que ces règlements ne seraient promulgués que dans un an .  </s>
<s snum=0266> je estime que nous ne avons pas les moyens de nous payer plus longtemps une pareille façon de se comporter à le cabinet .  </s>
<s snum=0267> ce est donc dire , monsieur le Président , que le gouvernement se doit de respecter les recommandations de le Commissaire à les langues officielles .  </s>
<s snum=0268> ensuite , ils réduiront le nombre de les emplois car les entreprises que ils auront achetées feront double emploi avec de autres que ils ont déjà .  </s>
<s snum=0269> ce est à la deuxième étape de réexamen que la Cour suprême a établi que une audience est obligatoire .  </s>
<s snum=0270> en ce qui concerne les avocats spécialisés en immigration , le barreau se charge de les pénaliser .  </s>
<s snum=0271> le autre solution consiste à investir sous forme de prêts .  </s>
<s snum=0272> veut - il attendre que le économie de ces régions agricoles soit complètement à terre avant de réagir à le problème de les pluies acides ?  </s>
<s snum=0273> il me semble que cela ne est pas compliqué à comprendre .  </s>
<s snum=0274> nous ne devons pas oublier toutefois que la majorité de les producteurs de boeuf et de porc ont carrément rejeté tout mécanisme de régularisation de les mises en marché .  </s>
<s snum=0275> si je avais été à la place de Bryce , je aurais partagé le assiette à le beurre avec les autres .  </s>
<s snum=0276> en un mot , nous considérons que le rôle politique et le rôle administratif doivent être séparés le un de le autre .  </s>
<s snum=0277> on le a négligé et liquidé , ce qui a obligé le gouvernement à investir pour faire absorber cet atelier .  </s>
<s snum=0278> tous ces endroits inclus dans notre patrimoine sont gérés par Parcs Canada à le profit et pour le agrément de tous les Canadiens .  </s>
<s snum=0279> comme je le ai dit tout à le heure , pareilles distinctions ne avaient pas été envisagées lorsque le projet de loi a été adopté en deuxième lecture .  </s>
<s snum=0280> les secteurs public et privé ont tous deux un rôle à remplir .  </s>
<s snum=0281> les critiques seront de autant plus vives que le gouvernement actuel , sauf erreur , se est engagé à agir à le grand jour .  </s>
<s snum=0282> ce ne est pas suffisant .  </s>
<s snum=0283> ce est lui qui nous a dit que la construction de le centre de entretien de VIA Rail à Montréal était remise à les calendes grecques .  </s>
<s snum=0284> cependant , M. Baldwin avait ensuite déclaré :  </s>
<s snum=0285> nous voulons que les promesses faites par les sociétés qui reprennent de les entreprises canadiennes soient révélées publiquement .  </s>
<s snum=0286> il faut également consulter les gouvernements provinciaux pour définir les besoins de certaines régions .  </s>
<s snum=0287> ils pensent que en changeant simplement le nom de AEIE en Investissement Canada , nous aurons un afflux de dollars .  </s>
<s snum=0288> est - ce que le ministre peut donner en cette Chambre la position de le gouvernement canadien ?  </s>
<s snum=0289> voici maintenant neuf mois que le nouveau gouvernement a été porté à le pouvoir , et nous attendons toujours son premier budget .  </s>
<s snum=0290> nous acceptons à le heure actuelle que soient enregistrées les associations canadiennes de sport amateur et il faudrait en faire autant pour les groupes provinciaux .  </s>
<s snum=0291> par définition , « association canadienne de sport amateur » se applique à les associations fédérales , mais non à leurs contreparties provinciales .  </s>
<s snum=0292> cette bataille a ravi à le Canada 1,800 vaillants marins et 24 fiers bâtiments .  </s>
<s snum=0293> je trouve incroyable que le président de le Conseil de le Trésor se permette de rire de un sujet aussi grave .  </s>
<s snum=0294> de plus , ils ont demandé de les budgets supplémentaires .  </s>
<s snum=0295> bon , tout le monde est de accord là - dessus .  </s>
<s snum=0296> le poste de directeur général a été vacant durant plus de un an , et pour cause .  </s>
<s snum=0297> le ministre a parlé de réunions , fort bien .  </s>
<s snum=0298> il fallait que il soit à Saint - Jean .  </s>
<s snum=0299> ce est exactement ce qui se passe .  </s>
<s snum=0300> il ne est pas question de les traiter séparément .  </s>
<s snum=0301> de toute évidence , il ne aime pas les gens qui appartiennent à le mouvement coopératif dans le est de le Canada .  </s>
<s snum=0302> il incombe à le gouvernement fédéral de jouer le rôle de agent de surveillance .  </s>
<s snum=0303> la Société veut que les employés à temps partiel constituent dorénavant 45 % de tous ses effectifs .  </s>
<s snum=0304> À le cours de les dernières élections provinciales à le Manitoba , 900 personnes ont voté par la poste ; surtout de les citadins .  </s>
<s snum=0305> alors le gros bon sens ce est quoi ?  </s>
<s snum=0306> les abattoirs bâtis selon la réglementation québécoise ont de les normes plus sévères .  </s>
<s snum=0307> les techniciens gagnent plus de argent que les écrivains et les auteurs de pièces de théâtre .  </s>
<s snum=0308> ils étaient tous gauchistes .  </s>
<s snum=0309> elles sont bien faites .  </s>
<s snum=0310> certes , nous demeurons un petit pays et nous sommes vulnérables pour cette raison .  </s>
<s snum=0311> m. Wyman compte de nombreuses années de expérience dans le secteur de les marchés financiers .  </s>
<s snum=0312> réfléchissons - y un petit instant .  </s>
<s snum=0313> il est inutile , je crois , de continuer à tourner autour de le pot pour régler la question .  </s>
<s snum=0314> cette façon de procéder permet à les Indiens de participer à le activité économique .  </s>
<s snum=0315> le un de les grands objectifs de les consultations est de faire en sorte que la relance profite également à tous .  </s>
<s snum=0316> je pense , monsieur le Président , que le discours de le député de Baie - Comeau était probablement de la fausse représentation .  </s>
<s snum=0317> la justice élémentaire exige que les emplois de les femmes soient évalués équitablement .  </s>
<s snum=0318> nous comptons poursuivre ces discussions en tirant parti de les opinions exprimées par de nombreux groupes intéressés .  </s>
<s snum=0319> je pense surtout à les mères célibataires .  </s>
<s snum=0320> le ministre de le Expansion économique régionale traite avec énormément de justice et de honnêteté les dossiers de le Québec .  </s>
<s snum=0321> en fait , le revenu agricole net était à son plus bas niveau depuis 1970 et à son troisième plus bas niveau depuis 1938 , soit en 45 ans .  </s>
<s snum=0322> dans la région de Laval - Deux - Montagnes , 225,000 dollars de budget en 1984 , 30,000 dollars seulement cette année .  </s>
<s snum=0323> ce même ministre a lui - même promis durant la campagne électorale que non seulement il consulterait les pêcheurs mais que il serait facilement accessible en cas de problème .  </s>
<s snum=0324> on néglige actuellement de investir dans ce domaine , surtout en Colombie - Britannique .  </s>
<s snum=0325> le pire , ce est que on ne a pas su obtenir une garantie de financement par la voie de une Entente de développement économique régional .  </s>
<s snum=0326> il devrait parler sincèrement à le lieu de nous servir tout ce charabia .  </s>
<s snum=0327> il a dit que British Telecom « ne se était guère montrée habile sur le plan de la commercialisation ou de la gestion de les produits » .  </s>
<s snum=0328> les investisseurs étrangers menaient le Canada comme se il se était agi de le cinquante - et - unième État .  </s>
<s snum=0329> elle ne fait pas de recherche et de développement .  </s>
<s snum=0330> la première question consiste à savoir pourquoi il faut stabiliser les prix dans le industrie agricole .  </s>
<s snum=0331> en règle générale , tous les producteurs de une denrée quelconque subissent en même temps le contrecoup de le resserrement de le étau coût - prix .  </s>
<s snum=0332> ce est donc une circonscription très diversifiée .  </s>
<s snum=0333> le véritable problème est donc que les agriculteurs abandonnent .  </s>
<s snum=0334> venons - en à ce qui semble être le opinion publique sur la réforme de le Sénat .  </s>
<s snum=0335> ce est ce que il faut et ce est le objectif visé par le gouvernement .  </s>
<s snum=0336> je pense que , indépendamment de notre parti , nous trouvons tous cela inacceptable .  </s>
<s snum=0337> revenons maintenant à le projet de loi en question .  </s>
<s snum=0338> ce que les conservateurs proposent de faire pour donner un caractère distinctif à leur projet de loi est une erreur .  </s>
<s snum=0339> et on sait comment cela peut être pénible pour de les gens qui vivent une période pas spécialement drôle de leur vie .  </s>
<s snum=0340> par conséquent , la création de un tel système devrait être la première priorité dans toutes les provinces où il ne en existe pas .  </s>
<s snum=0341> nous sommes collègues depuis longtemps .  </s>
<s snum=0342> elle aidera les sociétés canadiennes à redresser plus rapidement leur situation financière en attirant de nouveaux capitaux propres .  </s>
<s snum=0343> le universalité de les allocations familiales est maintenue .  </s>
<s snum=0344> la recherche ne est guère considérée comme prioritaire .  </s>
<s snum=0345> ah , bon ?  </s>
<s snum=0346> ainsi , le savon et les boissons pour les enfants coûteront davantage , de même que les aliments pour animaux domestiques , le essence et le chauffage domestique .  </s>
<s snum=0347> cette technique ne a rien de anormal .  </s>
<s snum=0348> de après le dernier budget libéral , de autre part , le revenu personnel avait augmenté de 7 % en 1984 .  </s>
<s snum=0349> le gouvernement dépense largement et renfloue les entreprises .  </s>
<s snum=0350> les femmes profiteront de nouveaux règlements concernant les prestations de survivant et le partage de les pensions entre les conjoints en cas de rupture de le mariage .  </s>
<s snum=0351> en réalité , si on tient compte de le Accord de le Ouest , il est plutôt probable que ce sera 2.8c . le litre .  </s>
<s snum=0352> nous respectons nos promesses électorales .  </s>
<s snum=0353> nous devons remonter le économie , créer de le emploi , rationaliser les dépenses de le État , administrer plus efficacement .  </s>
<s snum=0354> cela ne tient guère debout , surtout si le on regarde qui paye .  </s>
<s snum=0355> que feront - ils avec ce capital ?  </s>
<s snum=0356> cela équivaut à peu près à les 440 millions de dollars versés à Canadair en le espace de deux ans .  </s>
<s snum=0357> il est ressorti de ces audiences que la grande préoccupation était les gains en capital .  </s>
<s snum=0358> il donne à entendre à les particuliers que pendant de trop nombreuses années le gouvernement a décidé pour eux .  </s>
<s snum=0359> dans ce sens - là , je vous dirai que je ai rencontré dernièrement de les centaines de jeunes Canadiens , et ils ont perdu beaucoup de illusions .  </s>
<s snum=0360> ce est pourquoi je ne puis appuyer la demande de emprunt présentée par le gouvernement .  </s>
<s snum=0361> mon collègue dit « négligé le Québec » , je suis de accord avec lui aussi .  </s>
<s snum=0362> je suis heureux de apprendre que les sociétés de État qui ont une valeur commerciale mais ne répondent pas à les objectifs de la politique publique soient vendues .  </s>
<s snum=0363> lorsque nous avons préparé le budget , nous avons pris cela en considération .  </s>
<s snum=0364> le premier ministre respectera il cet engagement ?  </s>
<s snum=0365> le ministre pourrait - il préciser les effets de cette réduction ?  </s>
<s snum=0366> je ne ai parlé jusque ici que de les aspects géographiques de ma circonscription .  </s>
<s snum=0367> nous pensons que ils relèveront le défi .  </s>
<s snum=0368> le ministre est - il en mesure de affirmer que cette mesure est équitable ?  </s>
<s snum=0369> il y a longtemps que nos milieux de affaires ne ont pas reçu de un gouvernement un tel vote de confiance .  </s>
<s snum=0370> le parti progressiste conservateur de le Canada se y oppose .  </s>
<s snum=0371> ils se sont levés comme un seul homme pour applaudir bien fort .  </s>
<s snum=0372> huitièmement .  </s>
<s snum=0373> non , monsieur le Président , à le contraire .  </s>
<s snum=0374> je voudrais maintenant vous parler un peu de ce qui a motivé la suppression de le impôt sur les gains en capital .  </s>
<s snum=0375> cette initiative va augmenter la pauvreté chez les personnes les plus vulnérables , les personnes âgées , qui ne pourront pas trouver de autres sources de revenu .  </s>
<s snum=0376> en outre , sont stationnés sur cette base , les Snow Birds , le équipe de acrobatie aérienne de le Canada .  </s>
<s snum=0377> notre caucus a entendu le autre jour le Conseil économique de les provinces de le Atlantique .  </s>
<s snum=0378> faut pas nous prendre pour de les poires !  </s>
<s snum=0379> il se agit de un problème important .  </s>
<s snum=0380> je ai le intention de parler surtout aujourd' hui de les nombreuses améliorations apportées à les programmes de pensions de tous les Canadiens .  </s>
<s snum=0381> on estimait également que il fallait compenser le érosion de la valeur de les biens due à le inflation .  </s>
<s snum=0382> il démontre une attitude réaliste , également parce que il agit avec douceur et intelligence .  </s>
<s snum=0383> le gouvernement ne se y intéresse pas et il ne se en soucie pas .  </s>
<s snum=0384> ils ont voté pour un changement dans la gestion de les affaires de le pays .  </s>
<s snum=0385> monsieur le Président , un rappel à le Règlement .  </s>
<s snum=0386> les faits sont clairs et nets .  </s>
<s snum=0387> aussi la motion de le député de Beaches nous permet de faire en cette Chambre ce que tout le monde fait présentement à travers le pays .  </s>
<s snum=0388> de plus , ils ont une bibliothèque spécialisée à leur service et la société ERCO dépense près de deux millions chaque année pour ses services de recherche .  </s>
<s snum=0389> deuxièmement , il faut aider les entreprises canadiennes à tirer parti de les possibilités de investissement et de le développement technologique .  </s>
<s snum=0390> non seulement les risques sont plus élevés , mais le temps nécessaire pour préparer le réapprovisionnement par navire est long .  </s>
<s snum=0391> monsieur le Président , une fois encore , les prévisions pessimistes de les libéraux et de les néo - démocrates en matière économique ne se sont pas concrétisées .  </s>
<s snum=0392> je ne ai pas le intention de ennuyer la Chambre en répétant les mêmes réponses .  </s>
<s snum=0393> cependant , lorsque nous avons pris le pouvoir , nous avons découvert que la tendance était à la hausse .  </s>
<s snum=0394> rien ne oblige non plus un ministre à déposer autre chose que le document dont il a tiré une citation .  </s>
<s snum=0395> en outre , il est stipulé dans notre résolution que il faudrait avoir servi plus de 365 jours à le Canada pour être admissible .  </s>
<s snum=0396> le gouvernement avait alors prétendu que il ne avait pas eu le temps de discuter de ce projet avec le un ou le autre de ces deux groupes .  </s>
<s snum=0397> autrement dit , notre destin économique est scellé .  </s>
<s snum=0398> ils disaient que les propriétaires allaient encore écoper parce que les taux de intérêt augmentaient .  </s>
<s snum=0399> notre rôle sur la scène internationale est quelque peu éclipsé .  </s>
<s snum=0400> en ce qui concerne le assurance - récolte , par exemple , le gouvernement fédéral a contribué la moitié de les primes et les producteurs le autre moitié .  </s>
<s snum=0401> donc , il est impossible pour nous de penser en ces termes .  </s>
<s snum=0402> elle a déclaré que le seul problème que posent le Candu et toute le option nucléaire , ce est sur le plan de la commercialisation .  </s>
<s snum=0403> comment se faisait - il que le on me présentait toutes ces instances ?  </s>
<s snum=0404> » ce est cela que le actuel premier ministre avait dit à le ancien premier ministre .  </s>
<s snum=0405> la majorité sont de ailleurs incapables de travailler .  </s>
<s snum=0406> il y a quelques jours , mon collègue de Essex - Windsor a donné à la Chambre une leçon de histoire .  </s>
<s snum=0407> voici le texte de le article :  </s>
<s snum=0408> il se chauffait à le mazout .  </s>
<s snum=0409> le gouvernement ne a pas encore décidé comment et quand cette modification devrait se manifester .  </s>
<s snum=0410> je ai parlé tantôt de le pari de le premier ministre ( M. Mulroney ) concernant le emploi en immigration .  </s>
<s snum=0411> cependant , le projet de loi dont nous sommes saisis aujourd' hui ne contient que deux amendements de fond et un certain nombre de amendements de ordre administratif .  </s>
<s snum=0412> il ne nous est pas permis en tant que société de faire ce genre de discrimination entre les gens en fonction de un pareil critère .  </s>
<s snum=0413> une collaboration entre le gouvernement fédéral et les provinces est indispensable pour accroître la valeur et le efficacité de nos programmes de dépistage .  </s>
<s snum=0414> selon lui , il vaudrait mieux attendre à le moins jusque en juin .  </s>
<s snum=0415> le entreprise , tout en demeurant dans le secteur privé , fut rebaptisée en 1934 la Société de les transports de le nord Limitée .  </s>
<s snum=0416> je félicite sincèrement le ministre de les Transports de avoir respecté le mandat initial .  </s>
<s snum=0417> ce ne est que aujourd' hui que le secteur privé fait savoir que il ne est pas en mesure de soutenir la concurrence de toutes ces sociétés de État .  </s>
<s snum=0418> les services spécialisés , comme le ravitaillement de le secteur pétrolier et gazier de la mer de Beaufort , pourraient être assurés par de autres .  </s>
<s snum=0419> malheureusement ou heureusement , selon le point de vue de chacun , ce ministre ne a guère obtenu de succès .  </s>
<s snum=0420> toutefois , la présidence pourrait peut - être en terminer avec la motion no 1 .  </s>
<s snum=0421> sur la côte ouest , nous avons le comité consultatif de le ministre qui , même se il ne est pas parfait , peut à le moins être utile .  </s>
<s snum=0422> voilà un excellent exemple de association fructueuse entre les gouvernements et le industrie .  </s>
<s snum=0423> monsieur le Président , je ai une question à poser à le ministre de les Transports .  </s>
<s snum=0424> monsieur le Président , je ne veux pas que il y ait de malentendu .  </s>
<s snum=0425> À le sein de le gouvernement de le Canada , divers ministères ont joué un rôle actif et sont intervenus dans le cadre de une action coordonnée et coopérative .  </s>
<s snum=0426> toutefois , la contradiction qui existe entre sa déclaration et celle de ses hauts fonctionnaires devrait faire le objet de une enquête indépendante .  </s>
<s snum=0427> je sais que nous nous en occupons à le niveau fédéral par le intermédiaire de la Société de le crédit agricole .  </s>
<s snum=0428> tels sont les principaux indices économiques .  </s>
<s snum=0429> le établissement de une sixième exploitation agricole à Bowden , en Alberta , permettra de accroître encore la production agricole de les établissements carcéraux .  </s>
<s snum=0430> tel était le principe de ce projet de loi à le étape de la deuxième lecture .  </s>
<s snum=0431> certes , personne , et surtout pas un seul député conservateur , ne y croirait .  </s>
<s snum=0432> ce est ce que elle a fait le 4 septembre dernier .  </s>
<s snum=0433> Convoquez les députés .  </s>
<s snum=0434> nous croyons que il est temps de établir de les règlements et de les méthodes comme il se doit .  </s>
<s snum=0435> leur intervention était tout à fait justifiée , car le succès de ces accords dépendra dans une très large mesure de la qualité de leur administration .  </s>
<s snum=0436> le Canada a tout lieu de être fier de son bilan dans ce domaine et devrait servir de modèle de tolérance pour les autres pays .  </s>
<s snum=0437> sur le avis de qui a il pris cette décision , puisque ses services avaient refusé ?  </s>
<s snum=0438> monsieur le Président , je suis heureuse que le premier ministre ait fourni ces informations à la Chambre .  </s>
<s snum=0439> je prie donc le député de cesser de montrer quoi que ce soit .  </s>
<s snum=0440> pourquoi le premier ministre se est - il montré prêt à manquer à sa promesse en premier lieu ?  </s>
<s snum=0441> monsieur le Président , ces questions inquisitrices de le parti libéral me font toujours sourire .  </s>
<s snum=0442> assurément , le député ne voudrait pas que le on impose de les sociétés qui perdent de le argent .  </s>
<s snum=0443> monsieur le Président , la question de le député part de une hypothèse toute gratuite .  </s>
<s snum=0444> combien de personnes sont employées a ) directement ou à contrat , b ) à temps plein et à temps partiel à le Bureau de le premier ministre ?  </s>
<s snum=0445> les cachets versés à chacun étaient conformes à le rôle joué pendant le gala et respectaient les normes de le Union de les artistes .  </s>
<s snum=0446> sous le gouvernement libéral , rien ne a été fait pour remédier à cette situation .  </s>
<s snum=0447> ils connaissent très bien le problème de surproduction .  </s>
